#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>

#include <linux/dvb/dmx.h>
#include <linux/dvb/frontend.h>

static int cont = 1;

static void stop_loop(int sig)
{
	cont = 0;
	return;
}

int main(int argc, char **argv)
{
	int fe, fd, ffd = 0;
	struct sigaction act;
	struct dmx_pes_filter_params flt;
	int wfd;
	unsigned char buf[512];
	int adapter = 0;
	int channel = 20;
	int seconds = 0;
	int opt, len;
	struct dvb_frontend_parameters param;
	char aname[64];
	char fname[64];
	char dname[64];

	while ((opt = getopt(argc, argv, "a:c:t:")) != -1) {
		switch (opt) {
		case 'a':
			adapter = strtoul(optarg, NULL, 0);
			break;
		case 'c':
			channel = strtoul(optarg, NULL, 0);
			break;
		case 't':
			seconds = strtoul(optarg, NULL, 0);
			break;
		default:
			fprintf(stderr, "Usage: %s [-a <adapter#>] [-c <physical ch.#>] [-t <length(sec.)>] [<out-file>]\n", argv[0]);
			return -1;
		}
	}

	if (optind >= argc) {
		wfd = fileno(stdout);
	} else {
		wfd = open(argv[optind], O_WRONLY|O_CREAT|O_TRUNC, 0644);
		if (wfd < 0) {
			perror("failed to open the output-file.");
			return -wfd;
		}
	}

	snprintf(aname, sizeof(aname), "/dev/dvb/adapter%i/dvr0", adapter);
	snprintf(fname, sizeof(fname), "/dev/dvb/adapter%i/frontend0", adapter);
	snprintf(dname, sizeof(dname), "/dev/dvb/adapter%i/demux0", adapter);

	fd = open(aname, O_RDONLY);
	if (fd < 0) {
		close(wfd);
		return -fd;
	}
	ioctl(fd, DMX_SET_BUFFER_SIZE, 8*1024 * 1024);

	param.frequency = (473 + 6 * (channel - 13)) * 1000000;
	fe = open(fname, O_RDWR);
	if (fe < 0
	    || ioctl(fe, FE_SET_FRONTEND_TUNE_MODE, FE_TUNE_MODE_ONESHOT)
	    || ioctl(fe, FE_SET_FRONTEND, &param) < 0) {
		close(fd);
		return -fe;
	}
	usleep(500*1000);
	close (fe);


	ffd = open(dname, O_RDONLY);
	if (ffd < 0) {
		perror(dname);
		close(fd);
		close(fe);
		return -ffd;
	}

	ioctl(ffd, DMX_SET_BUFFER_SIZE, 16*1024*1024);
	flt.input = DMX_IN_FRONTEND;
	flt.output = DMX_OUT_TS_TAP;
	flt.pid = 0x2000;
	flt.pes_type = DMX_PES_OTHER;
	flt.flags = 0;

	if (ioctl(ffd, DMX_SET_PES_FILTER, &flt) < 0) {
		perror("DMX_SET_PES_FILTER");
		goto bailout;
	}


	sigemptyset(&act.sa_mask);
	act.sa_handler = stop_loop;
	act.sa_flags = 0;
	sigaction(SIGINT, &act, NULL);
	if (seconds) {
		sigaction(SIGALRM, &act, NULL);
		alarm(seconds);
	}

	if (ioctl(ffd, DMX_START, 1) < 0) goto bailout;
	while (cont) {
		len = read(fd, buf, sizeof(buf));
		if (len <= 0) break;
		if ( write(wfd, buf, len) != len ) break;
	}
	ioctl(ffd, DMX_STOP, 1);
bailout:
	close(ffd);
	close(fe);
	close(wfd);
	close(fd);
	return 0;
}

