#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <stdint.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include <linux/dvb/dmx.h>

static const char *usage = "\nusage: %s [-a adapter_num] [-d demuxer_id] [-t length(sec.)]\n\n";

int
main(int argc, char **argv)
{
	struct dmx_pes_filter_params flt;
	int adapter = 0, demuxer = 0;
	int demuxer_fd;
	int opt;
	unsigned int wtime = UINT32_MAX;
	char DEMUXER_DEV[64];

	while ((opt=getopt(argc, argv, "hrn:a:f:d:c:t:")) != -1) {
		switch (opt) {
		case 'a':
			adapter = strtoul(optarg, NULL, 0);
			break;
		case 'd':
			demuxer = strtoul(optarg, NULL, 0);
			break;
		case 't':
			wtime = strtoul(optarg, NULL, 0);
			break;
		case 'f':
		case 'r':
		case 'c':
			break; // for the compatibility with {t,s,a}zap
		case '?':
		case 'h':
		default:
			fprintf(stderr, usage, argv[0]);
			return -1;
		};
	}

	snprintf(DEMUXER_DEV, sizeof(DEMUXER_DEV),
		 "/dev/dvb/adapter%i/demux%i", adapter, demuxer);

	printf("using '%s'\n", DEMUXER_DEV);

	if ((demuxer_fd = open(DEMUXER_DEV, O_RDONLY)) < 0) {
		perror("failed to opening the demuxer device.");
		return -1;
	}

	/* may be useless, setting the buffer of the filter, not of the DVR */
	ioctl(demuxer_fd, DMX_SET_BUFFER_SIZE, 8*1024*1024);

	flt.input = DMX_IN_FRONTEND;
	flt.output = DMX_OUT_TS_TAP;
	flt.pid = 0x2000;
	flt.pes_type = DMX_PES_OTHER;
	flt.flags = DMX_IMMEDIATE_START;
	if (ioctl(demuxer_fd, DMX_SET_PES_FILTER, &flt) < 0) {
		perror("ioctl DMX_SET_PES_FILTER failed.");
		close(demuxer_fd);
		return -1;
	}

	sleep(wtime);

	if (ioctl(demuxer_fd, DMX_STOP, 1) < 0) {
		perror("ioctl DMX_STOP failed.");
		close(demuxer_fd);
		return -1;
	}
	close(demuxer_fd);
	return 0;
}
