/*
 *  Header file ISDB-T/S PSI data structures
 *
 *  Copyright (C) 2008 0p1pp1 
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.=
 */
#ifndef _ISDB_PSI_H_
#define _ISDB_PSI_H_

#include "isdb-demulti2.h"

struct secbuf {
	int cc;
	int max;
	int len;
	int cur;
	u8 *buf;
};


struct ca {
	u16 cas_id;
	u16 ca_pid;
	u8  type;
};


/* an ecm_info is shared among the programs with the same ECM pid */
/*   and it's kept for later reuse even when no program refers to it */
struct ecm_info {
	u16 pid;
	u8  ref_cnt;  /* 0: not used/free, 1+: in use  */
	u8  ver;      /* versino of the section */

	struct demulti2_key_info kinfo;  /* isdb-demulti2.h */
};


struct pmt {
	int ver;
	u16 prog_id;
	u16 pcr_pid;
	struct ca prog_ecm;
	struct ecm_info *ecm_ref;  /* (constant) ref to struct ecm_table */
	int ecm_idx;

#define ISDB_PSI_MAX_ES 16
	int num_es;
	struct {
#define ES_TYPE_MPEG1 0x01
#define ES_TYPE_MPEG2 0x02
#define ES_TYPE_TXT   0x06
#define ES_TYPE_AAC   0x0F
#define ES_TYPE_H264  0x1B
		u8 es_type;
		u8 tag; /* link with EIT */
		u16 es_pid;
		struct ca es_ecm; /* only the ecm_pid of 0x1fff is allowed */
	} es[ISDB_PSI_MAX_ES];

	struct secbuf sec;
	uint8_t sbuf[512];
};

struct program {
	u16 prog_id;
	u16 pmt_pid;
	struct pmt pmt;
};

struct pat {
	int ver;
	u16 ts_id;
	int num_prog;
#define ISDB_PSI_MAX_PROG 8
	struct program prog[ISDB_PSI_MAX_PROG];

	struct ecm_info ecmt[ISDB_PSI_MAX_PROG];

	struct secbuf sec;
	u8 sbuf[376];  /* 188 shoud be enough */
};


struct emm_info {
	struct ca ca;
	int ver;

	struct secbuf sec;
	u8 sbuf[4096];
};

struct cat {
	int ver;
	int num_cas;
#define ISDB_PSI_MAX_CA 2
	struct emm_info emm[ISDB_PSI_MAX_CA];

	struct secbuf sec;
	u8 sbuf[376];
};



#define TID_PAT 0x00
#define TID_CAT 0x01
#define TID_PMT 0x02
#define TID_ST 0x72
#define TID_ECM 0x82
#define TID_EMM 0x84

/* TS packet, 2nd Byte */
#define TS_F_TSEI 0x80
#define TS_F_PUSI 0x40
#define TS_F_PRIO 0x20

/* TS packet, 4th Byte */
#define TS_F_ADAP 0x20
#define TS_F_PAYLOAD 0x10
#define TS_CA_ODD  0xC0
#define TS_CA_EVEN 0x80
#define TS_CA_MASK 0x80
#define TS_CA_FLAG 0xC0
#define TS_CA_REV_MASK 0x3f
#define TS_CA_OE_MASK 0x40
#define TS_CC_MASK 0x0F

#define PID_PAT 0x0000
#define PID_CAT 0x0001
#define PID_NONE 0x1fff
#define PID_ALL 0x2000

/* see ARIB-STD-B10 part1 5.3 */
#define DESC_CA 0x09
#define DESC_SERVICE_LIST 0x41
#define DESC_SERVICE 0x48
#define DESC_EVENT_SHORT 0x4D
#define DESC_COMPONENT_V 0x50
#define DESC_STREAM_ID 0x52
#define DESC_COMPONENT_A 0xC4
#define DESC_TS_INFO 0xCD
#define DESC_PARTIAL_RECV 0xFB


#define VER_NONE -1
#define ID_NONE 0xffff

/* ARIB TR-B14/15 */
#define ID_CAS_ARIB 0x0005
/* ARIB TR-B26 */
#define ID_CAS_ARIB_B 0x000A

#define CA_EMM_TYPE_A 0x01
#define CA_EMM_TYPE_B 0x02


#endif /* _ISDB_PSI_H */
