/*
 *  Driver for JDVBT90502 Tuner Module Frontend
 *
 *  Copyright (C) 2008 0p1pp1 
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.=
 */

#ifndef JDVBT90502_H
#define JDVBT90502_H

#include <linux/dvb/frontend.h>
#include "dvb_frontend.h"

struct jdvbt90502_config {
	/* i2c addr for demodulator IC */
	u8 demod_address;

	/* i2c addr for tuner IC connected to the secondary i2c bus */
	u8 pll_address;
};

/* register through which the secondary I2C bus (PLL) is accessed */
#define JDVBT90502_2ND_I2C_REG 0xFE

/* byte index for pll i2c command data structure*/
/* see datasheet for tua6034 */
#define DEMOD_REDIRECT_REG 0
#define ADDRESS_BYTE       1
#define DIVIDER_BYTE1      2
#define DIVIDER_BYTE2      3
#define CONTROL_BYTE       4
#define BANDSWITCH_BYTE    5
#define AGC_CTRL_BYTE      5
#define PLL_CMD_LEN        6

/* bit masks for PLL STATUS response */
#define PLL_STATUS_POR_MODE   0x80 /* 1: Power on Reset (test) Mode */
#define PLL_STATUS_LOCKED     0x40 /* 1: locked */
#define PLL_STATUS_AGC_ACTIVE 0x08 /* 1:active */
#define PLL_STATUS_TESTMODE   0x07 /* digital output level(5 level), 0.15Vcc step, 0x00: < 0.15Vcc, ..., 0x04: >= 0.6Vcc (<= 1Vcc) */


#if defined(CONFIG_DVB_JDVBT90502) || (defined(CONFIG_DVB_JDVBT90502_MODULE) && defined(MODULE))
extern struct dvb_frontend *jdvbt90502_attach(
				const struct jdvbt90502_config *conf,
				struct i2c_adapter *i2c);
#else
extern inline struct dvb_frontend *jdvbt90502_attach(
				const struct jdvbt90502_config *conf,
				struct i2c_adapter *i2c)
{
	printk(KERN_WARNING "%s: driver disabled by Kconfig\n", __func__);
	return NULL;
}
#endif /* CONFIG_DVB_JDVBT90502 */

static inline int jdvbt90502_write(struct dvb_frontend *fe, u8 *buf, int len) {
	int r = 0;
	if (fe->ops.write)
		r = fe->ops.write(fe, buf, len);
	return r;
}

#endif // DVB_DUMMY_FE_H
